@@ -1308,7 +1308,6 @@ int ssl_hook_Fixup(request_rec *r)
  *
  * So we generated 512 and 1024 bit temporary keys on startup
  * which we now just handle out on demand....
- *
  */
 RSA *ssl_callback_TmpRSA(SSL *pSSL, int nExport, int nKeyLen)
 {
@@ -1319,16 +1318,16 @@ RSA *ssl_callback_TmpRSA(SSL *pSSL, int nExport, int nKeyLen)
     if (nExport) {
         /* It's because an export cipher is used */
         if (nKeyLen == 512)
-            rsa = mc->pRSATmpKey512;
+            rsa = (RSA *)mc->pTmpKeys[SSL_TKPIDX_RSA512];
         else if (nKeyLen == 1024)
-            rsa = mc->pRSATmpKey1024;
+            rsa = (RSA *)mc->pTmpKeys[SSL_TKPIDX_RSA1024];
         else
             /* it's too expensive to generate on-the-fly, so keep 1024bit */
-            rsa = mc->pRSATmpKey1024;
+            rsa = (RSA *)mc->pTmpKeys[SSL_TKPIDX_RSA1024];
     }
     else {
         /* It's because a sign-only certificate situation exists */
-        rsa = mc->pRSATmpKey1024;
+        rsa = (RSA *)mc->pTmpKeys[SSL_TKPIDX_RSA1024];
     }
     return rsa;
 }
@@ -1345,16 +1344,16 @@ DH *ssl_callback_TmpDH(SSL *pSSL, int nExport, int nKeyLen)
     if (nExport) {
         /* It's because an export cipher is used */
         if (nKeyLen == 512)
-            dh = mc->pDHTmpParam512;
+            dh = (DH *)mc->pTmpKeys[SSL_TKPIDX_DH512];
         else if (nKeyLen == 1024)
-            dh = mc->pDHTmpParam1024;
+            dh = (DH *)mc->pTmpKeys[SSL_TKPIDX_DH1024];
         else
             /* it's too expensive to generate on-the-fly, so keep 1024bit */
-            dh = mc->pDHTmpParam1024;
+            dh = (DH *)mc->pTmpKeys[SSL_TKPIDX_DH1024];
     }
     else {
         /* It's because a sign-only certificate situation exists */
-        dh = mc->pDHTmpParam1024;
+        dh = (DH *)mc->pTmpKeys[SSL_TKPIDX_DH1024];
     }
     return dh;
 }
