@@ -270,22 +270,13 @@ BOOL ssl_mutex_file_release(void)
 **  _________________________________________________________________
 */
 
-#if defined(SSL_CAN_USE_SEM) && defined(SSL_HAVE_IPCSEM) && defined(NEED_UNION_SEMUN)
-/* some platforms lack the `union semun' definition. Tz... */
-union semun {
-    long val;
-    struct semid_ds *buf;
-    unsigned short int *array;
-};
-#endif
-
 void ssl_mutex_sem_create(server_rec *s, pool *p)
 {
 #ifdef SSL_CAN_USE_SEM
     int semid;
     SSLModConfigRec *mc = myModConfig();
 #ifdef SSL_HAVE_IPCSEM
-    union semun semctlarg;
+    union ssl_ipc_semun semctlarg;
     struct semid_ds semctlbuf;
 #endif
 
